@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 Support for the 'format' key in the swagger spec as outlined in
 https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#dataTypeFormat
@@ -5,18 +6,19 @@ https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#dataType
 import functools
 from collections import namedtuple
 
-import six
 import dateutil.parser
+import pytz
+import six
 
 from bravado_core import schema
-
+from bravado_core.exception import SwaggerMappingError
 
 if six.PY3:
     long = int
 
 
 def NO_OP(x):
-    return None
+    return x
 
 
 def to_wire(swagger_spec, primitive_spec, value):
@@ -28,12 +30,23 @@ def to_wire(swagger_spec, primitive_spec, value):
     :param value: primitive to convert to wire representation
     :type value: int, long, float, boolean, string, unicode, object, etc
     :rtype: int, long, float, boolean, string, unicode, etc
+    :raises: SwaggerMappingError when format.to_wire raises an exception
     """
     if value is None or not schema.has_format(swagger_spec, primitive_spec):
         return value
     format_name = schema.get_format(swagger_spec, primitive_spec)
     formatter = swagger_spec.get_format(format_name)
-    return formatter.to_wire(value) if formatter else value
+
+    try:
+        return formatter.to_wire(value) if formatter else value
+    except Exception as e:
+        raise SwaggerMappingError(
+            'Error while marshalling value={} to type={}{}.'.format(
+                value, primitive_spec['type'],
+                '/{}'.format(primitive_spec['format']) if 'format' in primitive_spec else '',
+            ),
+            e,
+        )
 
 
 def to_python(swagger_spec, primitive_spec, value):
@@ -53,7 +66,7 @@ def to_python(swagger_spec, primitive_spec, value):
 
 
 class SwaggerFormat(namedtuple('SwaggerFormat',
-                    'format to_python to_wire validate description')):
+                               'format to_python to_wire validate description')):
     """User-defined format which can be registered with a
     :class:`bravado_core.spec.Spec` to handle marshalling to wire format,
     unmarshalling to a python type, and format specific validation.
@@ -111,7 +124,7 @@ DEFAULT_FORMATS = {
         description='Converts [wire]number:double <=> python float'),
     'date-time': SwaggerFormat(
         format='date-time',
-        to_wire=lambda dt: dt.isoformat(),
+        to_wire=lambda dt: (dt if dt.tzinfo else pytz.utc.localize(dt)).isoformat(),
         to_python=lambda dt: dateutil.parser.parse(dt),
         validate=NO_OP,  # jsonschema validates date-time
         description=(
@@ -134,4 +147,4 @@ DEFAULT_FORMATS = {
         to_python=lambda i: i if isinstance(i, long) else long(i),
         validate=NO_OP,  # jsonschema validates integer
         description='Converts [wire]integer:int64 <=> python long'),
-    }
+}
