@@ -1,13 +1,20 @@
+# -*- coding: utf-8 -*-
 from six import iteritems
 
-from bravado_core import formatter, schema
+from bravado_core import formatter
+from bravado_core import schema
 from bravado_core.exception import SwaggerMappingError
-from bravado_core.model import is_model, MODEL_MARKER
+from bravado_core.model import is_model
+from bravado_core.model import is_object
+from bravado_core.model import Model
+from bravado_core.model import MODEL_MARKER
+from bravado_core.schema import collapsed_properties
+from bravado_core.schema import get_spec_for_prop
+from bravado_core.schema import handle_null_value
 from bravado_core.schema import is_dict_like
 from bravado_core.schema import is_list_like
 from bravado_core.schema import is_prop_nullable
 from bravado_core.schema import SWAGGER_PRIMITIVES
-from bravado_core.schema import get_spec_for_prop
 
 
 def marshal_schema_object(swagger_spec, schema_object_spec, value):
@@ -27,7 +34,7 @@ def marshal_schema_object(swagger_spec, schema_object_spec, value):
     """
     deref = swagger_spec.deref
     schema_object_spec = deref(schema_object_spec)
-    obj_type = schema_object_spec['type']
+    obj_type = schema_object_spec.get('type')
 
     if obj_type in SWAGGER_PRIMITIVES:
         return marshal_primitive(swagger_spec, schema_object_spec, value)
@@ -46,7 +53,7 @@ def marshal_schema_object(swagger_spec, schema_object_spec, value):
         # key for identification.
         return marshal_model(swagger_spec, schema_object_spec, value)
 
-    if obj_type == 'object':
+    if is_object(swagger_spec, schema_object_spec):
         return marshal_object(swagger_spec, schema_object_spec, value)
 
     if obj_type == 'file':
@@ -73,9 +80,8 @@ def marshal_primitive(swagger_spec, primitive_spec, value):
         default_used = True
         value = schema.get_default(swagger_spec, primitive_spec)
 
-    if value is None and schema.is_required(swagger_spec, primitive_spec):
-        raise SwaggerMappingError(
-            'Spec {0} is a required value'.format(primitive_spec))
+    if value is None:
+        return handle_null_value(swagger_spec, primitive_spec)
 
     if not default_used:
         value = formatter.to_wire(swagger_spec, primitive_spec, value)
@@ -92,6 +98,9 @@ def marshal_array(swagger_spec, array_spec, array_value):
     :rtype: list
     :raises: SwaggerMappingError
     """
+    if array_value is None:
+        return handle_null_value(swagger_spec, array_spec)
+
     if not is_list_like(array_value):
         raise SwaggerMappingError('Expected list like type for {0}: {1}'
                                   .format(type(array_value), array_value))
@@ -119,26 +128,33 @@ def marshal_object(swagger_spec, object_spec, object_value):
     """
     deref = swagger_spec.deref
 
+    if object_value is None:
+        return handle_null_value(swagger_spec, object_spec)
+
     if not is_dict_like(object_value):
         raise SwaggerMappingError('Expected dict like type for {0}:{1}'.format(
             type(object_value), object_value))
 
+    object_spec = deref(object_spec)
+    required_fields = object_spec.get('required', [])
+    properties = collapsed_properties(object_spec, swagger_spec)
+
     result = {}
     for k, v in iteritems(object_value):
 
         prop_spec = get_spec_for_prop(
-            swagger_spec, deref(object_spec), object_value, k)
+            swagger_spec, object_spec, object_value, k, properties)
 
-        # Values cannot be None unless x-nullable is set
-        is_nullable = prop_spec and is_prop_nullable(swagger_spec, prop_spec)
-        if v is None and not is_nullable:
-            continue
-
-        if prop_spec:
-            result[k] = marshal_schema_object(swagger_spec, prop_spec, v)
-        else:
+        if not prop_spec:
             # Don't marshal when a spec is not available - just pass through
             result[k] = v
+            continue
+
+        if v is None and k not in required_fields:
+            if not is_prop_nullable(swagger_spec, prop_spec):
+                continue
+
+        result[k] = marshal_schema_object(swagger_spec, prop_spec, v)
 
     return result
 
@@ -159,16 +175,18 @@ def marshal_model(swagger_spec, model_spec, model_value):
     if model_type is None:
         raise SwaggerMappingError('Unknown model {0}'.format(model_name))
 
-    if not isinstance(model_value, model_type):
+    if model_value is None:
+        return handle_null_value(swagger_spec, model_spec)
+
+    if not isinstance(model_value, Model):
         raise SwaggerMappingError(
-            'Expected model of type {0} for {1}:{2}'
-            .format(model_name, type(model_value), model_value))
+            'Expected {0.__module__}.{0.__name__} object but got {1.__module__}.{1.__name__}'.format(
+                Model, type(model_value),
+            ),
+        )
 
     # just convert the model to a dict and feed into `marshal_object` because
     # models are essentially 'type':'object' when marshaled
-    attr_names = dir(model_value)
-    object_value = dict(
-        (attr_name, getattr(model_value, attr_name))
-        for attr_name in attr_names)
+    object_value = model_value._as_dict()
 
     return marshal_object(swagger_spec, model_spec, object_value)
