@@ -1,10 +1,13 @@
-from collections import defaultdict
+# -*- coding: utf-8 -*-
 import logging
+from collections import defaultdict
 
 from six import iteritems
 
 from bravado_core.exception import SwaggerMappingError
 from bravado_core.operation import Operation
+from bravado_core.util import AliasKeyDict
+from bravado_core.util import sanitize_name
 
 log = logging.getLogger(__name__)
 
@@ -47,7 +50,7 @@ def build_resources(swagger_spec):
     # key = tag_name   value = { operation_id : Operation }
     tag_to_ops = defaultdict(dict)
     deref = swagger_spec.deref
-    spec_dict = deref(swagger_spec.spec_dict)
+    spec_dict = deref(swagger_spec._internal_spec_dict)
     paths_spec = deref(spec_dict.get('paths', {}))
     for path_name, path_spec in iteritems(paths_spec):
         path_spec = deref(path_spec)
@@ -69,9 +72,11 @@ def build_resources(swagger_spec):
             for tag in tags:
                 tag_to_ops[deref(tag)][op.operation_id] = op
 
-    resources = {}
+    resources = AliasKeyDict()
     for tag, ops in iteritems(tag_to_ops):
-        resources[tag] = Resource(tag, ops)
+        sanitized_tag = sanitize_name(tag)
+        resources[sanitized_tag] = Resource(sanitized_tag, ops)
+        resources.add_alias(tag, sanitized_tag)
     return resources
 
 
@@ -83,8 +88,9 @@ class Resource(object):
     :param ops: operations associated with this resource (by tag)
     :type ops: dict where (key, value) = (op_name, Operation)
     """
+
     def __init__(self, name, ops):
-        log.debug(u"Building resource '%s'" % name)
+        log.debug(u"Building resource '%s'", name)
         self.name = name
         self.operations = ops
 
